<?php

// --------------------------------------------------------------------------
// Récupère les adresses des adhérents dans la base de données,
// récupère les coordonnées géographiques correspondantes à l'aide du service
// de géocodage de l'API Google Map et met à jour la table coord_adherents
// et les enregistre dans la table galette_dynamic_fields.
// Les coordonnées sont ainsi accessibles dans les fiches adhérents de galette.
// Silvain Yart et Gérald Yart
// --------------------------------------------------------------------------

// -- Modifications --
// -- 26/02/2012 --
// > Enregistrement des coordonnées dans les champs personnalisés de galette (table galette_dynamic_fields)
//   pour permettre les modifications par l'interface galette.
// > Géocodage uniquement pour les adhérents dont les coordonnées ne figurent pas déjà dans la table galette_dynamic_fields.
//   Evite d'écraser les modifications manuelles.
// -- 24/02/2012 --
// > En cas d'erreur de géocodage le message retourne le nom de l'adhérent plutôt que son numéro (qui ne peut pas être retrouvé dans l'interface galette)
// > Géocodage des adresses d'adhérents actifs uniquement (activite_adh = 1)

// Définition des constantes
// Serveur Google Maps auquel on adresse la requête
define("MAPS_HOST", "maps.google.fr");
// Clé API Google Maps
define("KEY", "Mettre ici votre Clé API Google Maps");

// Récupération des informations de connexion à la base de données (fichier de configuration de Galette)
include('../galette/includes/config.inc.php');

// Connexion à la base de données
$connect = mysql_connect(HOST_DB, USER_DB, PWD_DB);
if(!$connect){
	die('Connexion à la base de données impossible : '.mysql_error());
}

//Sélection de la base de données
$db = mysql_select_db(NAME_DB, $connect);
if(!$db){
	die('Sélection de la base de données impossible : '.mysql_error());
}

// Suppression des coordonnées qui ne correspondent plus à des adhérents actifs.
$query = "DELETE FROM galette_dynamic_fields 
	      WHERE (field_id = '4' 
		  OR field_id = '5') 
		  AND item_id NOT IN (
							  SELECT id_adh
							  FROM galette_adherents
							  WHERE activite_adh = '1'
							  )";
$result = mysql_query($query);
if (!$result) {
  die("L'exécution de la requête a retourné une erreur : " . mysql_error());
}
$nb_efface = mysql_affected_rows();


// Sélection des adresses dans la table galette_adherents
$query = "SELECT id_adh, nom_adh, prenom_adh, adresse_adh, adresse2_adh, cp_adh, ville_adh, pays_adh 
		  FROM galette_adherents 
		  WHERE activite_adh = '1'
		  AND id_adh NOT IN (
							 SELECT DISTINCT item_id
							 FROM galette_dynamic_fields
							 WHERE field_id = '4'
							 AND item_id IN (
											 SELECT DISTINCT item_id
											 FROM galette_dynamic_fields
											 WHERE field_id = '5'
											 )
							 )";
$result = mysql_query($query);
if (!$result) {
  die("L'exécution de la requête a retourné une erreur : " . mysql_error());
}

// Les requêtes successives ne doivent pas être envoyées trop rapidement au géocodeur.
// Un code d'erreur est retourné si les requêtes successives sont trop rapprochées.
// On définit donc un délai à respecter entre chaque requête.
// Ce délai est initialement fixé à 0 et augmenté à chaque fois qu'une erreur est retournée.
// La requête qui a échoué est envoyée une nouvelle fois avec le délai augmenté
$delay = 0;
$base_url = "http://".MAPS_HOST."/maps/geo?output=xml"."&key=".KEY;


// Comptage des géocodage réussis/échoués
$nb_succes = 0;
$nb_erreur = 0;
$msg_erreur = '';
$msg_succes = '';

// On parcourt chaque ligne renvoyée par la requête pour géocoder chaque adresse
while($row=@mysql_fetch_assoc($result)){
	// Le geocodage de l'adresse est défini initialement commen "en attente"
	$geocode_pending = true ;
	
	//Tant que le géocodage n'est pas effectué on recommence l'opération
	while($geocode_pending){
		//On regroupe les différents parties de l'adresse dans une seule variable
		$address = $row["adresse_adh"]." ".$row["adresse2_adh"]." ".$row["cp_adh"]." ".$row["ville_adh"]." ".$row["pays_adh"];
    	$id = $row["id_adh"];
		$nom = $row["prenom_adh"].' '.$row["nom_adh"];
		// On met en forme l'URL de la requête
    	$request_url = $base_url . "&q=" . urlencode($address);
		// Et on anvoie la requête dont on récupère le résultat dans une variable
    	$xml = simplexml_load_file($request_url) or die("url not loading");
		
		// Analyse de la réponse ...
		$status = $xml->Response->Status->code;
		if(strcmp($status, "200")==0){
			// Le géocodage a réussi : on récupère les coordonées dans des variables
			$geocode_pending = false ;
			$coordinates = $xml->Response->Placemark->Point->coordinates;
			$coordinatesSplit = split(",",$coordinates);
			//Format : Longitude, Latitude, Altitude
			$lat = $coordinatesSplit[1];
			$lng = $coordinatesSplit[0];
			
			// On met à jour la base de données avec les coordonnées récupérées
			$query = "REPLACE INTO galette_dynamic_fields (item_id, field_id, field_form, val_index, field_val) VALUES ('".$id."', '4', 'adh', '1', '".$lat."'), ('".$id."', '5', 'adh', '1', '".$lng."')";
			$insert_result = mysql_query($query);
			if(!$insert_result){
				die("L'exécution de la requête a retourné une erreur : " . mysql_error());
			}
			$nb_succes ++;
			$msg_succes .= $nom." : lat : ".$lat."° ; lon : ".$lng."° <br/>\n";
		} elseif(strcmp($status,"620")==0){ // Si la requête retourne le code d'erreur 620 (requêtes successives trop rapides)
			// On augmente le délai entre deux requêtes
			$delay += 100000;
		} else { // Si la requête retourne un autre code d'erreur
			// on abandonne et on affiche un message d'erreur
			$geocode_pending = false ;
			$nb_erreur ++;
			$msg_erreur .= $nom." - ".$address." - Erreur ".$status." <br/>\n";
		}
		usleep($delay);
	}
}

$message = "<b>Opération terminée :<br>";
if($nb_efface > 1){
	$message .= "- ".$nb_efface." localisations ont été effacées<br>";
}elseif($nb_efface == 1){
	$message .= "- ".$nb_efface." localisation a été effacée<br>";
}
if($nb_succes > 1){
	$message .= "- ".$nb_succes." adresses ont été géocodées avec succès :</b><br>
				".$msg_succes."<br>";
}elseif($nb_succes == 1){
	$message .= "- ".$nb_succes." adresse a été géocodée avec succès :</b><br>
				".$msg_succes."<br>";
}else{
	$message .= "- Aucune adresse n'a été géocodée</b><br>";
}

if($nb_erreur > 1){
	$message .= "<b>- ".$nb_erreur." adresses n'ont pas pu être géocodées :</b><br>
				".$msg_erreur."<br>
				Plus d'infos sur les codes d'erreur <a href=\"http://code.google.com/intl/fr/apis/maps/documentation/javascript/v2/reference.html#GGeoStatusCode\">ici</a>";
}elseif($nb_erreur == 1){
	$message .= "<b>- ".$nb_erreur." adresse n'a pas pu être géocodée :</b><br>
				".$msg_erreur."<br><br>
				Plus d'infos sur les codes d'erreur <a href=\"http://code.google.com/intl/fr/apis/maps/documentation/javascript/v2/reference.html#GGeoStatusCode\">ici</a><br>";
}else{
	$message .= "<b>- Aucune adresse n'a pas pu être géocodée</b><br>";
}
	
$message .= "<br><b>ATTENTION : seules les nouvelles adresses de la table galette_adherents ont été géocodées.<br>
				Pour modifier les coordonnées géographiques d'un adhérent dont l'adresse a déjà été géocodée il faut le faire manuellement via la fiche adhérent galette</b><br>";

mysql_close($connect);

?>
<!DOCTYPE html >
  <head>
    <meta name="viewport" content="initial-scale=1.0, user-scalable=no" />
    <meta http-equiv="content-type" content="text/html; charset=UTF-8"/>
    <title>Carte des membres</title>
  </head>
  <body>
  <?php echo $message; ?>
  </body>
</html
